///////////////////////////////////////////////////////////
//  Document.cs
//  Implementation of the Class Document
//  Generated by Enterprise Architect
//  Created on:      17-Feb-W008 0:53:17
///////////////////////////////////////////////////////////




using System.Collections.Generic;
using System;
using System.Data.Linq.Mapping;
using System.Data;
using System.Linq;
using System.Data.Linq;
using System.Drawing;
using HomeDmsUtils;
using System.Text;


namespace HomeDMSObjectModel 
{
	[Table(Name="Document")]
    public class Document 
    {
		private string _name;

        private int _documentId;

        [Column(Name="document_name")]
		public virtual string Name{
			get{
				return _name;
			}
			set{
				_name = value;
			}
		}
		private string _description;
		private DateTime? _from;
		private DateTime? _to;
        private int _lcid;
        private DateTime? _enteredOn = DateTime.Now;

        [Column(Name="document_id",IsPrimaryKey=true,DbType="Int identity(1,1)",IsDbGenerated=true)]
        public virtual int DocumentId
        {
            get { return _documentId; }
            set { _documentId = value; }
        }

        public virtual IList<Tag> Tags
        {
            get {
                HomeDMSContext db = DBContextFactory.GetCurrentContext() as HomeDMSContext;
                if (db != null)
                {
                    var q = from assoc in db.documentTags
                            join tag in db.tags on assoc.TagId equals tag.TagId
                            where assoc.DocumentId == this.DocumentId
                            select tag;
                    return q.ToList<Tag>();
                }
                else
                {
                    return new List<Tag>();
                }
            }
            
        }

        [Column(Name="document_description")]
		public virtual string Description{
			get{
				return _description;
			}
			set{
				_description = value;
			}
		}

		[Column(Name="document_from")]
        public virtual DateTime? From{
			get{
				return _from;
			}
			set{
				_from = value;
			}
		}

        private EntitySet<Page> _pages = new EntitySet<Page>();

        [Association(Name = "FK_PageDoc",Storage="_pages",OtherKey = "DocumentId")]
        public virtual EntitySet<Page> Pages
        {
            get{
                return _pages;
            }
            set
            {
                _pages.Assign(value);
            }
        }
        
        [Column(Name="document_to")]
		public virtual DateTime? To{
			get{
				return _to;
			}
			set{
				_to = value;
			}
		}

        [Column]
        public virtual DateTime? EnteredOn
        {
            get { return _enteredOn; }
            set { _enteredOn = value; }
        }        

        [Column]
        public virtual int LCID
        {
            get { return _lcid; }
            set { _lcid = value; }
        }

        public override bool Equals(object obj)
        {
            Document _doc = obj as Document;
            if (_doc == null)
                return false;
            else
                return _doc.DocumentId == this.DocumentId;
        }

        public override int GetHashCode()
        {
            return this.DocumentId.GetHashCode();
        }

        public virtual Image PresentPicture 
        {
            get
            {
                if (Pages == null || Pages.Count == 0)
                {
                    return null;
                }

                Image retValue;
                var q = from p in Pages where p.IsDefault select p;

                if (q.Count<Page>() > 0)
                {
                    retValue = q.First<Page>().Image;
                }
                else
                {
                    retValue = Pages[0].Image;
                }

                Size size = new Size(40, 40);
                return retValue.BuildThumbnailImage(size);
                //return null;
            }
        }


        public string Keywords {
            get
            {
                IList<Tag> tags = Tags;
                if (tags != null && tags.Count > 0)
                {
                    StringBuilder sb = new StringBuilder();
                    int counter = 0;
                    foreach (Tag t in tags)
                    {
                        if (counter == 0)
                        {
                            sb.Append(t.Name);
                        }
                        else
                        {
                            sb.AppendFormat(", {0}", t.Name);
                        }
                        counter++;
                    }
                    return sb.ToString();
                }
                else
                {
                    return string.Empty;
                }
            }
        }


	}//end Document

}//end namespace HomeDMS